/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.p4java.extension.server;

import java.util.List;

import com.perforce.p4java.extension.exception.ServerAccessException;
import com.perforce.p4java.extension.exception.ServerConnectionException;
import com.perforce.p4java.extension.exception.ServerErrorException;

/**
 * A pool of connections to the Perforce server.
 */
public interface ConnectionPool<T extends Server> {

    /**
     * Provides a Perforce server to the requestor.
     * 
     * @return a server from the pool
     */
    T acquire();

    /**
     * 
     * Provide a Perforce server to the caller after connecting with the
     * provided credential.
     * 
     * @param user
     *            non-null PrincipalCredential containing the Perforce user name
     *            and password
     * @return a suitable connected server from the pool
     * 
     * @throws ServerAccessException
     *             if the credential is not valid
     * @throws ServerConnectionException
     *             if there are issues connecting to the server
     * @throws ServerErrorException
     *             if the server has a problem processing the login request
     */
    T acquire(PrincipalCredential user) throws ServerAccessException,
	    ServerConnectionException, ServerErrorException;

    /**
     * Cleans are returns a server to the pool.
     * 
     * @param server
     *            the server to release
     */
    void release(T server);

    /**
     * Disconnect all server and remove them from pool.
     * <p/>
     * This is for test only
     */
    void releaseAllConnections();

    /**
     * return connection still connected
     * 
     * @return
     */
    List<T> getConnected();

    /**
     * return connection still connected and in aquire pool
     * 
     * @return
     */
    List<T> getAquired();
}
